# -*- coding: utf-8 -*-
from pip_services4_components.config import ConfigParams
from pip_services4_components.refer import References, Descriptor
from pip_services4_data.query import PagingParams, FilterParams

from eic_aichat_models.basemodels.data import BaseModel
from eic_aichat_models.basemodels.logic import BaseModelsService
from eic_aichat_models.basemodels.persistence import BaseModelsMemoryPersistence

BASE_MODEL1 = BaseModel(
    id='1',
    api='ai',
    name="name1",
    base_url="localhost:3000",
    supported_models=['model1', 'model2'],
)

BASE_MODEL2 = BaseModel(
    id='2',
    api='openapi',
    name="name2",
    base_url="localhost:3001",
    supported_models=['model3', 'model4'],
)


class TestBaseModelsService:
    persistence: BaseModelsMemoryPersistence
    service: BaseModelsService

    def setup_method(self):
        self.persistence = BaseModelsMemoryPersistence()
        self.persistence.configure(ConfigParams())

        self.service = BaseModelsService()
        self.service.configure(ConfigParams())

        references = References.from_tuples(
            Descriptor('aichatmodels-basemodels', 'persistence', 'memory', 'default', '1.0'), self.persistence,
            Descriptor('aichatmodels-basemodels', 'service', 'default', 'default', '1.0'), self.service
        )

        self.service.set_references(references)

        self.persistence.open(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_crud_operations(self):
        # Create the first model
        model = self.service.create_model(None, BASE_MODEL1)
        assert BASE_MODEL1.api == model.api
        assert BASE_MODEL1.base_url == model.base_url
        assert model.supported_models is not None

        # Create the second model
        model = self.service.create_model(None, BASE_MODEL2)
        assert BASE_MODEL2.api == model.api
        assert BASE_MODEL2.base_url == model.base_url
        assert model.supported_models is not None

        # Get all models
        page = self.service.get_models(None, FilterParams(), PagingParams())
        assert page is not None
        assert len(page.data) == 2

        model1: BaseModel = page.data[0]

        # Update the model
        model1.api = 'ABC'

        model = self.service.update_model(None, model1)
        assert model1.id == model.id
        assert 'ABC' == model.api

        # Get model by api
        model = self.service.get_model_by_api(None, model1.api)
        assert model1.id == model.id

        # Delete the model
        model = self.service.delete_model_by_id(None, model1.id)
        assert model1.id == model.id

        # Try to get deleted model
        model = self.service.get_model_by_id(None, model1.id)
        assert model is None



# -*- coding: utf-8 -*-

from pip_services4_commons.convert import TypeCode
from pip_services4_data.validate import ObjectSchema


class BaseModelSchema(ObjectSchema):
    def __init__(self):
        super().__init__()

        self.with_optional_property('id', TypeCode.String)
        self.with_required_property('name', TypeCode.String)
        self.with_required_property('api', TypeCode.String)
        self.with_optional_property('base_url', TypeCode.String)
        self.with_optional_property('supported_models', TypeCode.Array)



from typing import Optional

from pip_services4_components.config import ConfigParams, IConfigurable
from pip_services4_components.context import IContext
from pip_services4_components.refer import IReferences, IReferenceable, Descriptor
from pip_services4_data.keys import IdGenerator
from pip_services4_data.query import DataPage, PagingParams, FilterParams

from ..data import BaseModel
from ..persistence import IBaseModelsPersistence

from .IBaseModelsService import IBaseModelsService


class BaseModelsService(IBaseModelsService, IConfigurable, IReferenceable):
    __persistence: IBaseModelsPersistence = None

    def configure(self, config: ConfigParams):
        pass

    def set_references(self, references: IReferences):
        self.__persistence = references.get_one_required(Descriptor('aichatmodels-basemodels', 'persistence', '*', '*', '1.0'))

    def get_models(self, context: Optional[IContext], filter_params: FilterParams,
                   paging: PagingParams) -> DataPage:
        return self.__persistence.get_page_by_filter(context, filter_params, paging)

    def get_model_by_id(self, context: Optional[IContext], model_id: str) -> BaseModel:
        return self.__persistence.get_one_by_id(context, model_id)

    def get_model_by_api(self, context: Optional[IContext], api: str) -> BaseModel:
        return self.__persistence.get_one_by_api(context, api)
    
    def get_model_by_name(self, context: Optional[IContext], name: str) -> BaseModel:
        return self.__persistence.get_one_by_name(context, name)

    def create_model(self, context: Optional[IContext], model: BaseModel) -> BaseModel:
        model.id = model.id or IdGenerator.next_long()

        return self.__persistence.create(context, model)

    def update_model(self, context: Optional[IContext], model: BaseModel) -> BaseModel:

        return self.__persistence.update(context, model)

    def delete_model_by_id(self, context: Optional[IContext], model_id: str) -> BaseModel:
        return self.__persistence.delete_by_id(context, model_id)


from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, PagingParams, FilterParams

from ..data import BaseModel


class IBaseModelsService(ABC):

    def get_models(self, context: Optional[IContext], filter_params: FilterParams, paging: PagingParams) -> DataPage:
        raise NotImplementedError("Method is not implemented")

    def get_model_by_id(self, context: Optional[IContext], model_id: str) -> BaseModel:
        raise NotImplementedError("Method is not implemented")

    def get_model_by_api(self, context: Optional[IContext], api: str) -> BaseModel:
        raise NotImplementedError("Method is not implemented")
    
    def get_model_by_name(self, context: Optional[IContext], name: str) -> BaseModel:
        raise NotImplementedError("Method is not implemented")

    def create_model(self, context: Optional[IContext], model: BaseModel) -> BaseModel:
        raise NotImplementedError("Method is not implemented")

    def update_model(self, context: Optional[IContext], model: BaseModel) -> BaseModel:
        raise NotImplementedError("Method is not implemented")

    def delete_model_by_id(self, context: Optional[IContext], model_id: str) -> BaseModel:
        raise NotImplementedError("Method is not implemented")


# -*- coding: utf-8 -*-
from pip_services4_components.refer import Descriptor
from pip_services4_components.build import Factory

from eic_aichat_models.basemodels.logic.BaseModelsService import BaseModelsService
from eic_aichat_models.basemodels.persistence.BaseModelsMemoryPersistence import BaseModelsMemoryPersistence
from eic_aichat_models.basemodels.persistence.BaseModelsMongoDbPersistence import BaseModelsMongoDbPersistence


class AIChatModelsFactory(Factory):
    __MemoryPersistenceDescriptor = Descriptor('aichatmodels-basemodels', 'persistence', 'memory', '*', '1.0')
    __MongoDbPersistenceDescriptor = Descriptor('aichatmodels-basemodels', 'persistence', 'mongodb', '*', '1.0')
    __ServiceDescriptor = Descriptor('aichatmodels-basemodels', 'service', 'default', '*', '1.0')


    def __init__(self):
        super().__init__()

        self.register_as_type(self.__MemoryPersistenceDescriptor, BaseModelsMemoryPersistence)
        self.register_as_type(self.__MongoDbPersistenceDescriptor, BaseModelsMongoDbPersistence)
        self.register_as_type(self.__ServiceDescriptor, BaseModelsService)



# -*- coding: utf-8 -*-
from pip_services4_data.query import FilterParams, PagingParams

from eic_aichat_models.basemodels.data.BaseModel import BaseModel
from eic_aichat_models.basemodels.persistence.IBaseModelsPersistence import IBaseModelsPersistence

BASE_MODEL1 = BaseModel(
    id='1',
    api='ai',
    name="name1",
    base_url="localhost:3000",
    supported_models=['model1', 'model2'],
)

BASE_MODEL2 = BaseModel(
    id='2',
    api='openapi',
    name="name2",
    base_url="localhost:3001",
    supported_models=['model3', 'model4'],
)

BASE_MODEL3 = BaseModel(
    id='3',
    api='glhf',
    name="name3",
    base_url="localhost:3002",
    supported_models=['model5'],
)


class BaseModelsPersistenceFixture:
    _persistence: IBaseModelsPersistence

    def __init__(self, persistence: IBaseModelsPersistence):
        assert persistence is not None
        self._persistence = persistence

    def test_create_base_models(self):
        # Create the first model
        entity = self._persistence.create(None, BASE_MODEL1)
        assert BASE_MODEL1.api == entity.api
        assert BASE_MODEL1.base_url == entity.base_url
        assert entity.supported_models is not None

        # Create the second model
        entity = self._persistence.create(None, BASE_MODEL2)
        assert BASE_MODEL2.api == entity.api
        assert BASE_MODEL2.base_url == entity.base_url
        assert entity.supported_models is not None

        # Create the third model
        entity = self._persistence.create(None, BASE_MODEL3)
        assert BASE_MODEL3.api == entity.api
        assert BASE_MODEL3.base_url == entity.base_url
        assert entity.supported_models is not None

    def test_crud_operations(self):
        # Create items
        self.test_create_base_models()

        # Get all models
        page = self._persistence.get_page_by_filter(None, FilterParams(), PagingParams())
        assert page is not None
        assert len(page.data) == 3

        entity1: BaseModel = page.data[0]

        # Update the model
        entity1.api = 'ABC'

        entity = self._persistence.update(None, entity1)
        assert entity1.id == entity.id
        assert 'ABC' == entity.api

        # Get model by api
        entity = self._persistence.get_one_by_api(None, entity1.api)
        assert entity1.id == entity.id

        # Delete the model
        entity = self._persistence.delete_by_id(None, entity1.id)
        assert entity1.id == entity.id

        # Try to get deleted model
        entity = self._persistence.get_one_by_id(None, entity1.id)
        assert entity is None

    def test_get_with_filters(self):
        # Create items
        self.test_create_base_models()

        # Filter by id
        page = self._persistence.get_page_by_filter(None, FilterParams.from_tuples('id', '1'), PagingParams())
        assert len(page.data) != 0

        # Filter by supported_models
        page = self._persistence.get_page_by_filter(None,
                                                    FilterParams.from_tuples('supported_models', 'model1'),
                                                    PagingParams())
        assert len(page.data) != 0

        # Filter by api
        page = self._persistence.get_page_by_filter(None,
                                                    FilterParams.from_tuples('api', 'openapi'),
                                                    PagingParams())
        assert len(page.data) != 0



# -*- coding: utf-8 -*-
import json

import bottle
from pip_services4_commons.convert import TypeCode
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_data.validate import ObjectSchema
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_components.context import Context

from eic_aichat_models.basemodels.data import BaseModel, BaseModelSchema
from eic_aichat_models.basemodels.logic.IBaseModelsService import IBaseModelsService


class BaseModelsOperations(RestOperations):
    def __init__(self):
        super().__init__()
        self._base_models_service: IBaseModelsService = None
        self._dependency_resolver.put("basemodels-service", Descriptor('aichatmodels-basemodels', 'service', '*', '*', '1.0'))

    def configure(self, config):
        super().configure(config)

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._base_models_service = self._dependency_resolver.get_one_required('basemodels-service')

    def get_base_models(self):
        context = Context.from_trace_id(self._get_trace_id())
        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        try:
            res = self._base_models_service.get_models(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_base_model_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._base_models_service.get_model_by_id(context, id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def get_base_model_by_name(self, name):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._base_models_service.get_model_by_name(context, name)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def get_base_model_by_api(self, api):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._base_models_service.get_model_by_api(context, api)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def create_base_model(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json
        base_model = data if isinstance(data, dict) else json.loads(data)
        base_model = None if not base_model else BaseModel(**base_model)
        try:
            res = self._base_models_service.create_model(context, base_model)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def update_base_model(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json
        base_model = data if isinstance(data, dict) else json.loads(data)
        base_model = None if not base_model else BaseModel(**base_model)
        try:
            res = self._base_models_service.update_model(context, base_model)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def delete_base_model_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._base_models_service.delete_model_by_id(context, id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def register_routes(self, controller: RestController):
        # controller.register_route('get', '/base_models', ObjectSchema(True)
        #                           .with_optional_property("filter", FilterParamsSchema())
        #                           .with_optional_property("paging", PagingParamsSchema()),
        #                           self.get_base_models)
        controller.register_route('get', '/models/base', None,
                            self.get_base_models)

        controller.register_route('get', '/models/base/<id>', ObjectSchema(True)
                            .with_optional_property("base_model_id", TypeCode.String),
                            self.get_base_model_by_id)

        controller.register_route('post', '/models/base', ObjectSchema(True)
                            .with_required_property("body", BaseModelSchema()),
                            self.create_base_model)

        controller.register_route('put', '/models/base', ObjectSchema(True)
                            .with_required_property("body", BaseModelSchema()),
                            self.update_base_model)

        controller.register_route('delete', '/models/base/<id>', ObjectSchema(True)
                            .with_required_property("base_model_id", TypeCode.String),
                            self.delete_base_model_by_id)




# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, PagingParams, FilterParams

from ..data import BaseModel


class IBaseModelsPersistence(ABC):
    @abstractmethod
    def __init__(self):
        pass

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams, paging: PagingParams) -> DataPage:
        pass

    def get_one_by_id(self, context: Optional[IContext], entity_id: str) -> BaseModel:
        pass

    def get_one_by_api(self, context: Optional[IContext], api: str) -> BaseModel:
        pass

    def get_one_by_name(self, context: Optional[IContext], name: str) -> BaseModel:
        pass

    def create(self, context: Optional[IContext], entity: BaseModel) -> BaseModel:
        pass

    def update(self, context: Optional[IContext], entity: BaseModel) -> BaseModel:
        pass

    def delete_by_id(self, context: Optional[IContext], entity_id: str) -> BaseModel:
        pass



# -*- coding: utf-8 -*-
from typing import Dict, List
from pip_services4_data.data import IStringIdentifiable


class BaseModel(IStringIdentifiable):
    def __init__(self, id: str = None, name: str = None, api: str = None, base_url: str = None, supported_models: List[str] = None):
        self.id = id
        self.name = name
        self.api = api
        self.base_url = base_url
        self.supported_models = supported_models

    def to_dict(self) -> Dict[str, str]:
        return {
            'id': self.id,
            'name': self.name,
            'api': self.api,
            'base_url': self.base_url,
            'supported_models': ','.join(self.supported_models)
        }



# -*- coding: utf-8 -*-

__all__ = [
    'IBaseModelsPersistence', 'BaseModelsMemoryPersistence', 'BaseModelsMongoDbPersistence'
]

from .IBaseModelsPersistence import IBaseModelsPersistence
from .BaseModelsMemoryPersistence import BaseModelsMemoryPersistence
from .BaseModelsMongoDbPersistence import BaseModelsMongoDbPersistence



# -*- coding: utf-8 -*-
from eic_aichat_models.basemodels.persistence import BaseModelsMemoryPersistence
from test.basemodels.persistence.BaseModelsPersistenceFixture import BaseModelsPersistenceFixture


class TestBaseModelsMemoryPersistence:
    persistence: BaseModelsMemoryPersistence
    fixture: BaseModelsPersistenceFixture

    def setup_method(self):
        self.persistence = BaseModelsMemoryPersistence()

        self.fixture = BaseModelsPersistenceFixture(self.persistence)

        self.persistence.open(None)
        self.persistence.clear(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_crud_operations(self):
        self.fixture.test_crud_operations()

    def test_get_with_filters(self):
        self.fixture.test_get_with_filters()



# -*- coding: utf-8 -*-
from typing import Optional, Any

from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, PagingParams, FilterParams
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from ..data import BaseModel
from .IBaseModelsPersistence import IBaseModelsPersistence


class BaseModelsMongoDbPersistence(IdentifiableMongoDbPersistence, IBaseModelsPersistence):

    def __init__(self):
        super().__init__('base_models')

        self._max_page_size = 1000

    def __compose_filter(self, filter_params: FilterParams):
        filter_params = filter_params or FilterParams()

        filters = []

        id = filter_params.get_as_nullable_string('id')
        if id is not None:
            filters.append({'_id': id})

        api = filter_params.get_as_nullable_string('api')
        if api is not None:
            filters.append({'api': api})
        
        name = filter_params.get_as_nullable_string('name')
        if name is not None:
            filters.append({'name': name})

        temp_models = filter_params.get_as_nullable_string('supported_models')
        if temp_models is not None:
            models = temp_models.split(',')
            filters.append({'supported_models': {'$in': models}})

        return None if len(filters) < 1 else {'$and': filters}

    def get_page_by_filter(self, context: Optional[IContext], filter: Any, paging: PagingParams,
                           sort: Any = None, select: Any = None) -> DataPage:
        return super().get_page_by_filter(context, self.__compose_filter(filter), paging, None, None)

    def get_one_by_api(self, context: Optional[IContext], api: str) -> BaseModel:
        criteria = {'api': api}
        item = self._collection.find_one(criteria)

        if item is None:
            self._logger.trace(context, "Cannot find base model with api=%s", str(api))
        else:
            self._logger.trace(context, "Found base model with api=%s", str(api))

        item = self._convert_to_public(item)
        return item
    
    def get_one_by_name(self, context: Optional[IContext], name: str) -> BaseModel:
        criteria = {'name': name}
        item = self._collection.find_one(criteria)

        if item is None:
            self._logger.trace(context, "Cannot find base model with name=%s", str(name))
        else:
            self._logger.trace(context, "Found base model with name=%s", str(name))

        item = self._convert_to_public(item)
        return item



# -*- coding: utf-8 -*-
from typing import Optional, Any, Callable

from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, PagingParams, FilterParams, SortParams
from pip_services4_persistence.persistence import IdentifiableMemoryPersistence

from ..data import BaseModel
from .IBaseModelsPersistence import IBaseModelsPersistence


class BaseModelsMemoryPersistence(IdentifiableMemoryPersistence, IBaseModelsPersistence):

    def __init__(self):
        super().__init__()

        self._max_page_size = 1000

    def __compose_filter(self, filter_params: FilterParams) -> Callable[[BaseModel], bool]:
        filter_params = filter_params or FilterParams()

        id = filter_params.get_as_nullable_string('id')
        api = filter_params.get_as_nullable_string('api')
        supported_models = filter_params.get_as_nullable_string('supported_models')
        name = filter_params.get_as_nullable_string('name')
        names = filter_params.get_as_nullable_string('names')

        if isinstance(names, str):
            names = names.split(',')
        if not isinstance(names, list):
            names = None

        if isinstance(supported_models, str):
            supported_models = supported_models.split(',')
        if not isinstance(supported_models, list):
            supported_models = None

        def filter_action(item: BaseModel) -> bool:
            if id is not None and item.id != id:
                return False
            if api is not None and item.api != api:
                return False
            if name is not None and item.name != name:
                return False
            if names is not None and item.name not in names:
                return False
            if supported_models is not None:
                if not any(model in supported_models for model in item.supported_models):
                    return False
            return True

        return filter_action

    def get_page_by_filter(self, context: Optional[IContext], filter: FilterParams, paging: PagingParams,
                           sort: SortParams = None, select: Any = None) -> DataPage:
        return super().get_page_by_filter(context, self.__compose_filter(filter), paging, None, None)

    def get_one_by_api(self, context: Optional[IContext], api: str) -> BaseModel:
        filtered = list(filter(lambda item: item.api == api, self._items))
        item = None if len(filtered) < 1 else filtered[0]

        if item is None:
            self._logger.trace(context, "Cannot find base model with api=%s", str(api))
        else:
            self._logger.trace(context, "Found base model with api=%s", str(api))

        return item
    
    def get_one_by_name(self, context: Optional[IContext], name: str) -> BaseModel:
        filtered = list(filter(lambda item: item.name == name, self._items))
        item = None if len(filtered) < 1 else filtered[0]

        if item is None:
            self._logger.trace(context, "Cannot find base model with name=%s", str(name))
        else:
            self._logger.trace(context, "Found base model with name=%s", str(name))

        return item
